<?php

namespace Drupal\Tests\jqcloud\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;

/**
 * Tests the jQCloud block being displayed in the Home Page.
 *
 * @group jqcloud
 */
class JQCloudCreateBlockTest extends BrowserTestBase {

  /**
   * Modules to enable.
   *
   * @var array<string>
   */
  protected static $modules = ['jqcloud', 'taxonomy', 'block'];

  /**
   * Theme to enable.
   *
   * @var string
   */
  protected $defaultTheme = 'stark';

  /**
   * Disable strict config schema.
   *
   * @var bool
   */
  protected $strictConfigSchema = FALSE;

  /**
   * Block config storage.
   *
   * @var \Drupal\Core\Config\Entity\BlockEntityStorageInterface
   */
  protected $blockConfigStorage;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    // Make sure to complete the normal setup steps first.
    parent::setUp();

    $this->drupalLogin($this->drupalCreateUser([
      'administer taxonomy',
      'access taxonomy overview',
      'bypass node access',
      'administer blocks',
    ]));

    $vocab_id = "random_food";

    // Create test vocabulary.
    $vocabulary = Vocabulary::create([
      'vid' => $vocab_id,
      'description' => '',
      'name' => "Random food",
    ])->save();

    // Create terms for the vocabulary.
    $term = Term::create([
      'name' => 'Pizza',
      'vid' => $vocab_id,
    ])->save();

    $term = Term::create([
      'name' => 'Avocado',
      'vid' => $vocab_id,
    ])->save();

    $term = Term::create([
      'name' => 'Burritos',
      'vid' => $vocab_id,
    ])->save();

    // Clear all caches before looking if the new jQCloud block was created.
    drupal_flush_all_caches();
  }

  /**
   * Adds a few other blocks and tests the new jQCloud block visibility.
   */
  public function testBlockVisibility() {
    $assert_session = $this->assertSession();

    $this->drupalGet('/admin/structure/block');
    $assert_session->statusCodeEquals(200);
    $assert_session->pageTextContains('Block');
    $assert_session->pageTextContains('Header');
    $assert_session->responseContains('id="edit-blocks-region-header-title"');

    $this->drupalGet('/admin/structure/block/library/' . $this->defaultTheme);
    $assert_session->statusCodeEquals(200);
    $assert_session->pageTextContains('Block');
    $assert_session->pageTextContains('Category');
    $assert_session->pageTextContains('Operations');
    $assert_session->pageTextContains('jQCloud');

    // Add the new jQCloud random block in the content page.
    $this->drupalGet('/admin/structure/block/add/jqcloud_block%3Arandom_food/' . $this->defaultTheme);

    $assert_session->statusCodeEquals(200);
    $assert_session->buttonExists("Save block");

    $edit = [
      'region' => 'content',
    ];

    $this->submitForm($edit, 'Save block');
    $assert_session->pageTextContains('The block configuration has been saved.');

    $this->drupalGet('<front>');
    $assert_session = $this->assertSession();
    $assert_session->statusCodeEquals(200);

    // Check if the block title was rendered.
    $assert_session->pageTextContains('jQCloud with "Random food" vocabulary');

    // Check if the <div> jQCloud was rendered.
    $elements = $this->xpath('//div[contains(@class, "jqcloud-contents")]/*');
    $this->assertCount(1, $elements, "The jQCloud block was not rendered");
  }

  /**
   * Tests if the block has been loaded in the home page and the terms rendered.
   */
  public function testLoadHomePage() {
    // Load the front page.
    $this->drupalGet('<front>');

    $assert_session = $this->assertSession();

    // Confirm that the site didn't throw a server error or something else.
    $assert_session->statusCodeEquals(200);
  }

}
